#!/usr/bin/perl

# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

use strict;
use warnings;

use FindBin qw( $Bin );
use lib "$Bin/../lib";

use JSON qw( from_json to_json );

use AdvPowerMgr::Errors;
use AdvPowerMgr::I18N qw( i18n );
use AdvPowerMgr::Log;
use AdvPowerMgr::Log::File;
use AdvPowerMgr::System;

eval {
    AdvPowerMgr::Log->context("FixConfig");
    AdvPowerMgr::Log->default(AdvPowerMgr::Log::File->new());

    AdvPowerMgr::System->die_unless_root();
    die "This program should not be started manually!" unless defined($ARGV[0]);

    open(my $fh, '<', $ARGV[0]) or die_open($ARGV[0]);
    my $config = from_json(join('', (<$fh>)));
    close($fh);

    my @nodes = ( $config );
    my $node;
    while ($node = shift(@nodes)) {
        last if defined($node->{type});
        foreach my $v (values(%$node)) {
            push(@nodes, $v) if ref($v) eq "HASH";
        }
    }
    $node->{title} = i18n('package.displayname');
    $node->{title} .= " (TEST)" if AdvPowerMgr::System->testing();
    $node->{desc} = i18n('package.description');

    my $json = to_json($config);
    open($fh, '>', $ARGV[0]) or die_create($ARGV[0]);
    print $fh $json;
    close($fh);
};

if ($@) {
    my $error = $@;
    chomp($error);
    fatal($error);
    eval "require AdvPowerMgr::Home;";
    unless ($@) {
        eval {
            AdvPowerMgr::Home->post_report($error);
        };
        fatal($@) if $@;
    }
    print STDERR $error, "\n";
    exit(1);
}

exit(0);

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
