# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::CGI;

use strict;
use warnings;

BEGIN {
    use AdvPowerMgr::StdoutRedirect;
}

use JSON qw( to_json );

use AdvPowerMgr::Errors;
use AdvPowerMgr::Log;
use AdvPowerMgr::Log::Remote;

our $html;

sub header {
    my $class = shift;

    return <<EOF;
Content-Type: application/json

EOF
}

sub html {
    my $class = shift;
    my $text = shift or die_param('text');

    $html = $text;
}

our $fatal;

sub not_fatal {
    my $class = shift;

    $fatal = 0;
}

sub code;

sub die_code {
    my $class = shift;
    my $error = shift or die_param('error');

    $class->html(to_json({ success => JSON->false(), msg => $error }));
}

sub run {
    my $class = shift;

    $fatal = 1;

    my $log = AdvPowerMgr::Log::Remote->new();

    eval {
        AdvPowerMgr::Log->context(substr($class, rindex($class, "::") + 2));
        AdvPowerMgr::Log->default($log);
        eval {
            $class->code();
        };
        my $err = $@;
        if ($err) {
            $class->die_code($err);
            die $err;
        }
    };
    if ($@) {
        my $error = $@;
        chomp($error);
        if ($fatal) {
            fatal($error);
            eval "require AdvPowerMgr::Service::Notify; require AdvPowerMgr::Home;";
            unless ($@) {
                eval {
                    AdvPowerMgr::Service::Notify->call($error);
                    AdvPowerMgr::Home->post_report($error);
                };
                fatal($@) if $@;
            }
        } else {
            error($error);
        }
        $class->die_code($error);
    }

    AdvPowerMgr::StdoutRedirect->stop();
    my $stdout = AdvPowerMgr::StdoutRedirect->get();
    warning("Unexpected content on STDOUT: '%s'", $stdout) if defined($stdout);
    $log->sync();
    print $class->header();
    print $html;
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
