# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::CGI::I18N;

use strict;
use warnings;

BEGIN {
    use AdvPowerMgr::DropPrivileges;
    AdvPowerMgr::DropPrivileges->drop();
}

use base qw( AdvPowerMgr::CGI );

use JSON qw( to_json );

use AdvPowerMgr::Errors;
use AdvPowerMgr::HTTP;
use AdvPowerMgr::Home;
use AdvPowerMgr::I18N qw();
use AdvPowerMgr::Log;
use AdvPowerMgr::Service::I18NReset;
use AdvPowerMgr::Service::I18NSave;

sub code {
    my $class = shift;

    my $req = AdvPowerMgr::HTTP->parse({
        allowed_methods => [ 'GET', 'POST' ],
        allowed_params => {
            'GET' => [ 'op', 'lang', 'catalog' ],
            'POST' => [ 'lang' ]
        },
        allowed_data_length => 32768
    }) or return;
    my $params = $req->params();

    if ($req->method() eq 'GET') {
        my $ret = { success => JSON->true() };
        if (defined($params->{op})) {
            die_param("op") unless $params->{op} eq "list";
            $ret->{result} = [ AdvPowerMgr::I18N->list() ];
        } else {
            if (defined($params->{lang})) {
                # Used by tranlation panel
                my $catalog = $params->{catalog};
                my @langs = ( $params->{lang} );
                push(@langs, "en") if $catalog eq 'builtin';
                $ret->{result} = AdvPowerMgr::I18N->get({ langs => \@langs, catalog => $catalog });
            } else {
                # Used for UI localization
                $ret->{result} = AdvPowerMgr::I18N->get({ pattern => qr/^ui\./ }) or die_ie("LANG");
            }
        }
        $class->html(to_json($ret));
    } elsif ($req->method() eq 'POST') {
        my $lang = $params->{lang};
        die_param("lang") unless defined($lang);
        my $data = $req->data();
        foreach my $k (keys(%$data)) {
            delete $data->{$k} unless $data->{$k};
        }
        AdvPowerMgr::Service::I18NSave->call($lang, $data);
        my $err = "";
        if (%$data) {
            eval {
                AdvPowerMgr::Home->post_lang($lang, to_json($data));
            };
            $err = $@;
            warning("Failed to submit language to Home: %s", $err) if $err;
        }

        info("Resetting Scheduler messages");
        AdvPowerMgr::Service::I18NReset->call();

        $class->html(to_json({ success => JSON->true(), msg => $err }));
    }
};

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
