# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::CGI::Suspend;

use strict;
use warnings;

BEGIN {
    use AdvPowerMgr::DropPrivileges;
    AdvPowerMgr::DropPrivileges->drop();
}

use base qw( AdvPowerMgr::CGI );

use JSON qw( to_json );

use AdvPowerMgr::Plugin::PreShutdown::Suspend;
use AdvPowerMgr::Errors;
use AdvPowerMgr::HTTP;
use AdvPowerMgr::Log;
use AdvPowerMgr::Service::SchedulerReload;
use AdvPowerMgr::Service::SuspendSave;

sub code {
    my $class = shift;

    my $req = AdvPowerMgr::HTTP->parse({ allowed_methods => [ 'POST', 'GET' ], allowed_params => [], allowed_data_length => 8192 }) or return;

    if ($req->method() eq 'GET') {
        my $obj = AdvPowerMgr::Plugin::PreShutdown::Suspend->compute();
        $class->html(to_json({ success => JSON->true(), result => $obj }));
    } elsif ($req->method() eq 'POST') {
        my $data = $req->data();
        my $op = $data->{op};
        die_param("op") unless defined($op) && ($op =~ m/^(resume|suspend|delay)$/);
        my $obj;
        eval {
            $obj = AdvPowerMgr::Service::SuspendSave->call($data);
        };
        if ($@) { $class->not_fatal(); die $@; }

        info("Reloading Scheduler");
        AdvPowerMgr::Service::SchedulerReload->call();

        $class->html(to_json({ success => JSON->true(), result => $obj }));
    } else {
        die_ie("METHOD");
    }
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
