# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Class;

use strict;
use warnings;

use base qw( Exporter );

use Symbol qw( qualify_to_ref );

__PACKAGE__->_mk_accessors(qw( version disable_triggers ));

sub new {
    my $class = shift;
    my $params = defined($_[0]) ? { %{$_[0]} } : {}; shift;

    $params->{version} = 1 unless defined($params->{version});
    $params->{disable_triggers} = 0 unless defined($params->{disable_triggers});

    my $self = {};
    bless($self, $class);
    $self->disable_triggers($params->{disable_triggers});
    $self->version($params->{version});

    return $self;
}

sub _mk_accessors {
    my $class = shift;

    no strict 'refs';
    foreach my $field (@_) {
        my $ref = qualify_to_ref($field, $class);

        *{$ref} = sub {
            my $self = shift;
            if (@_) {
                # Setter
                my $val = shift;
                my $options = shift || {};
                my $func = "${field}_validator";
                if ($self->can($func) && !$options->{disable_validator}) {
                    $val = $self->$func($val);
                }
                my $was = $self->{$field};
                $self->{$field} = $val;
                $func = "${field}_trigger";
                if ($self->can($func) && !$options->{disable_trigger} && !$self->disable_triggers()) {
                    $self->$func($was);
                }
                return $was;
            } else {
                return $self->{$field};
            }
        };
    }
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
