# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Errors;

use strict;
use warnings;

use base qw( Exporter );

our @EXPORT_OK;
push(@EXPORT_OK, qw( die_ie die_param die_exec die_open die_create ));

our @EXPORT;
push(@EXPORT, @EXPORT_OK);

*CORE::GLOBAL::die = sub {
    my $str = shift || $@;
    chomp($str);
    my (undef, $filename, $line) = caller(1);
    eval "require AdvPowerMgr::Log;"; CORE::die if $@;
    AdvPowerMgr::Log->fatal("'%s' at %s on line %d", $str, $filename, $line);

    CORE::die "$str\n";
};

sub die_ie {
    shift if $_[0] eq __PACKAGE__;
    my $str = shift or die "Internal error: no 'str' specified";
    my $filename = shift;
    my $line = shift;

    unless (defined($filename) && defined($line)) {
        (undef, $filename, $line) = caller(1);
    }
    $str = "Internal error: $str";
    eval "require AdvPowerMgr::Log;"; CORE::die if $@;
    AdvPowerMgr::Log->fatal("'%s' at %s on line %d", $str, $filename, $line);
    CORE::die "$str\n";
}

sub die_param {
    shift if $_[0] eq __PACKAGE__;
    my $param = shift or die_ie("no 'param' specified");

    my (undef, $filename, $line) = caller(1);
    die_ie("no '$param' specified", $filename, $line);
}

sub die_exec {
    shift if $_[0] eq __PACKAGE__;
    my $cmd = shift or die_param("cmd");

    my (undef, $filename, $line) = caller(1);
    die_ie("failed to execute '$cmd': $!", $filename, $line);
}

sub die_open {
    shift if $_[0] eq __PACKAGE__;
    my $file = shift or die_param("file");

    my (undef, $filename, $line) = caller(1);
    die_ie("failed to open file '$file': $!", $filename, $line);
}

sub die_create {
    shift if $_[0] eq __PACKAGE__;
    my $file = shift or die_param("file");

    my (undef, $filename, $line) = caller(1);
    die_ie("failed to create file '$file': $!", $filename, $line);
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
