# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Log::Memory;

use strict;
use warnings;

use base qw( AdvPowerMgr::Log );

use AdvPowerMgr::Errors;

__PACKAGE__->_mk_accessors(qw( name ));
__PACKAGE__->_mk_accessors(qw( _arr ));

sub new {
    my $class = shift;
    my $params = defined($_[0]) ? { %{$_[0]} } : {}; shift;

    $params = { %$params, level => $class->maxlevel(), with_package => 1, with_time => 1, with_context => 1 };

    my $self = $class->SUPER::_new($params);
    bless($self, $class);
    $self->name($params->{name});
    $self->_arr([]);

    return $self;
}

sub _log {
    my $self = shift;
    my $params = shift or die_ie('params');

    push(@{$self->_arr()}, $params);
}

sub clear {
    my $self = shift;

    $self->_arr([]);

    return $self;
}

sub dump {
    my $self = shift;

    return @{$self->_arr()};
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
