# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Plugin;

use strict;
use warnings;

use base qw( AdvPowerMgr::Config::Component );

use JSON qw();

use AdvPowerMgr::Errors;
use AdvPowerMgr::Validators qw( boolean_validator );

__PACKAGE__->_mk_accessors(qw( enabled name ));
__PACKAGE__->_mk_accessors(qw( _laststate ));

sub enabled_validator {
    my $class = shift;
    my $value = shift;
    return boolean_validator($value);
}

sub enabled_trigger {
    my $self = shift;

    $self->_reset();
}

sub new {
    my $class = shift;
    my $params = defined($_[0]) ? { %{$_[0]} } : {}; shift;
    my $options = shift || {};

    if (defined($params->{objtype}) && $params->{objtype} ne $class) {
        die_ie(sprintf("unexpected objtype '%s'", $params->{objtype}));
    }

    $params->{enabled} = 0 unless defined($params->{enabled});
    $params->{name} = substr($class, rindex($class, "::") + 2) unless defined($params->{name});

    my $self = $class->SUPER::new($params, $options);
    bless($self, $class);
    $self->enabled($params->{enabled}, $options);
    $self->name($params->{name});
    return $self;
}

# Returns 'undef' if plugin must be called twice and first call (asynchronous plugin)
# Returns a hash with the following fields:
# - enabled
#   - 0 if plugin is disabled
#   - 1 otherwise
# - nextupdate_ts
#   - 0 if no further update required
#   - timestamp otherwise
sub compute {
    my $self = shift;
    my $when = shift || time();

    return $self->_laststate();
}

sub to_hash {
    my $self = shift;
    my $options = shift || {};

    my $ret = $self->SUPER::to_hash($options);
    if ($options->{for_json}) {
        $ret->{enabled} = $self->enabled() ? JSON->true() : JSON->false();
    } else {
        $ret->{enabled} = $self->enabled();
    }
    $ret->{name} = $self->name();

    return $ret;
}

sub _reset {
    my $self = shift;

    $self->_laststate({ enabled => $self->enabled(), nextupdate_ts => 0 });
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
