# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Plugin::PreWakeup::Hibernate;

use strict;
use warnings;

use base qw( AdvPowerMgr::Plugin::PreWakeup );

use Time::Local;

use AdvPowerMgr::Log;
use AdvPowerMgr::Validators qw();

__PACKAGE__->_mk_accessors(qw( date ));

sub date_validator {
    my $self = shift;
    my $value = shift;

    return $value unless defined($value);
    return AdvPowerMgr::Validators->date_validator($value);
}

sub new {
    my $class = shift;
    my $params = defined($_[0]) ? { %{$_[0]} } : {}; shift;
    my $options = shift || {};

    debug_func_enter($params);

    if (defined($params->{objtype}) && $params->{objtype} eq "AdvPowerMgr::Plugin::Hibernate") {
        # Upgrade from 3.x, x <= 2
        delete $params->{objtype};
    }

    my $self = $class->SUPER::new($params, { disable_trigger => 1 });
    bless($self, $class);
    $self->date($params->{date}, { disable_trigger => 1 });
    $self->_reset() unless exists($options->{disable_trigger}) && $options->{disable_trigger};
    return $self;
}

# compute() is expected to return a hash with the following fields:
# - enabled
#   - 0 if plugin is disabled
#   - 1 otherwise
# - nextupdate_ts
#   - 0
# - not_before_ts (valid if enabled only)
#   - 0 if no wakeup planned
#   - timestamp otherwise
sub compute {
    my $self = shift;
    my $when = shift || time();

    my $state = $self->_laststate();

    if ($self->enabled() && defined($self->date())) {
        my ($y, $m, $d) = split('-', $self->date());
        my $upto = timelocal(0, 0, 0, $d, $m - 1, $y - 1900);
        if ($when >= $upto) {
            delete $state->{not_before_ts};
            $state->{enabled} = 0;
        } else {
            $state->{not_before_ts} = $upto;
        }
    }

    return $state;
}

sub to_hash {
    my $self = shift;
    my $options = shift || {};

    my $ret = $self->SUPER::to_hash($options);
    $ret->{date} = $self->date();

    return $ret;
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
