# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::StdoutRedirect;

use strict;
use warnings;

use AdvPowerMgr::Errors;

our $buffer;
our ($real_stdout, $our_stdout, $selected);

open($our_stdout, '>', \$buffer);

sub start {
    my $class = shift;

    return if defined($selected) && $selected eq $our_stdout;
    die_ie("STDOUT") if defined($selected) && $selected ne $real_stdout;
    $real_stdout = select($our_stdout);
    $selected = $our_stdout;

    return $class;
}

sub started {
    my $class = shift;

    return 1 if defined($selected) && $selected eq $our_stdout;
    return 0;
}

sub stop {
    my $class = shift;

    return if defined($selected) && $selected eq $real_stdout;
    die_ie("STDOUT") if (defined($selected) && $selected ne $our_stdout) || !defined($real_stdout);
    select($real_stdout);
    $selected = $real_stdout;

    return $class;
}

sub get {
    my $class = shift;

    if (defined($buffer)) {
        return wantarray() ? split(/\n/, $buffer) : $buffer;
    } else {
        return wantarray() ? () : undef;
    }
}

sub clear {
    my $class = shift;

    my $started = $class->started();
    close($our_stdout);
    undef($buffer);
    open($our_stdout, '>', \$buffer);
    if ($started) {
        $selected = $our_stdout;
    }

    return $class;
}

__PACKAGE__->start();

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
