# Copyright (c) 2010-2015 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::System;

use strict;
use warnings;

use Symbol qw( qualify_to_ref );

use AdvPowerMgr::Errors;

{
    our $impl;

    if (defined($ENV{UNIT_TESTING})) {
        $impl = "AdvPowerMgr::System::UnitTesting";
    } elsif (! -e "/etc/synoinfo.conf") {
        $impl =  "AdvPowerMgr::System::Linux";
    } else {
        $impl = "AdvPowerMgr::System::Syno";
    }
    eval "require $impl;"; die if $@;

    $impl->init();

    foreach my $method ( qw(
        appdir
        appversion
        check_privilege
        die_unless_root
        get_arp_cache
        get_connections
        get_disk_stats
        get_networks
        get_network_stats
        hwid
        is_autobooting
        get_smartctl_disks
        lang
        notify
        proxy
        poweroff
        set_wakeup
        storedir
        testing
        tmpdir
    )) {
        my $ref = qualify_to_ref($method, $impl);
        die_ie("'$impl::$method' doesn't exist") unless defined($ref);
        *{qualify_to_ref($method, __PACKAGE__)} = sub {
            my $class = shift;
            return $impl->$method(@_);
        }
    }
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
