# Copyright (c) 2010-2014 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::System::Common;

use strict;
use warnings;

use Cwd qw( abs_path );

use AdvPowerMgr::Errors;

{
    our $appname;
    our $appdir;

    sub appdir {
        my $class = shift;

        $class->_appname() unless defined($appdir);
        return $appdir;
    }

    sub _appname {
        my $class = shift;

        unless (defined($appname)) {
            my $pkg = $class;
            s#::#/#g, s#$#.pm# for $pkg;
            my $fullpath = abs_path($INC{$pkg});
            $fullpath =~ m#^(.*/([^/]*))/lib/$pkg$#;
            $appdir = $1; $appname = $2;
            die_ie("failed to compute application directory") unless defined($appname);
        }
        return $appname;
    }
}

{
    our $version;

    sub appversion {
        my $class = shift;

        unless (defined($version)) {
            my $file = sprintf("%s/VERSION", $class->appdir());
            open(my $fh, '<', $file) or die_open($file);
            $version = <$fh>;
            close($fh);
            chomp($version);
        }
        return $version;
    }
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
