# Copyright (c) 2010-2015 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Utils;

use strict;
use warnings;

use base qw( Exporter );

use Time::Local qw( timelocal );

use AdvPowerMgr::Errors;

our @EXPORT_OK;
push(@EXPORT_OK, qw( min_ts max_ts time2str str2time ipv4_to_hex uniq millisleep txt2html ));

sub txt2html {
    my $class = shift if $_[0] eq __PACKAGE__;
    my $txt = shift;

    s#\n#<br/>#g for $txt;
    return $txt;
}

sub millisleep {
    my $class = shift if $_[0] eq __PACKAGE__;
    my $value = shift;

    select(undef, undef, undef, $value);
}

sub max_ts {
    my $class = shift if $_[0] eq __PACKAGE__;

    my @res = sort { $a <=> $b } @_;

    return ($res[0] == -1 ? -1 : $res[@res - 1]);
}

sub min_ts {
    my $class = shift if $_[0] eq __PACKAGE__;

    my @res = sort { $a <=> $b } @_;

    return -1 if $res[0] == -1;

    foreach (@res) {
        return $_ if $_ > 0;
    }

    return 0;
}

sub time2str {
    my $class = shift if $_[0] eq __PACKAGE__;
    my $ts = shift;

    return "None" if $ts == 0;
    return "Forever" if $ts == -1;

    my ($s, $m, $h, $dm, $mo, $y, $dw) = localtime($ts);
    $mo++; $y += 1900;

    return sprintf("%04d-%02d-%02d %02d:%02d:%02d", $y, $mo, $dm, $h, $m, $s);
}

{
    our $str2time_regex = qr/^(\d\d\d\d)-(\d\d)-(\d\d) (\d\d):(\d\d)(?::(\d\d))?$/;

    sub str2time {
        my $class = shift if $_[0] eq __PACKAGE__;
        my $date = shift or die_param("date");

        $date =~ $str2time_regex or die "Invalid date format '$date'";

        return timelocal($6 || 0, $5, $4, $3, $2 - 1, $1 - 1900);
    }
}

sub ipv4_to_hex {
    my $class = shift if $_[0] eq __PACKAGE__;
    my ($ip) = @_;
    return sprintf("%02x%02x%02x%02x", split(/\./, $ip));
}

sub uniq {
    return () unless @_;
    my $class = shift if $_[0] eq __PACKAGE__;

    my $prev = 'nonesuch';
    return grep($_ ne $prev && (($prev) = $_), @_);
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
